@@ -32,6 +32,7 @@
 #include "nmv-ui-utils.h"
 #include "nmv-i-workbench.h"
 #include "nmv-i-perspective.h"
+#include "nmv-debugger-utils.h"
 
 namespace nemiver {
 
@@ -43,13 +44,29 @@ struct BPColumns : public Gtk::TreeModelColumnRecord {
     Gtk::TreeModelColumn<Glib::ustring> function;
     Gtk::TreeModelColumn<int> line;
     Gtk::TreeModelColumn<Glib::ustring> condition;
+    Gtk::TreeModelColumn<bool> is_countpoint;
     Gtk::TreeModelColumn<Glib::ustring> type;
     Gtk::TreeModelColumn<int> hits;
     Gtk::TreeModelColumn<Glib::ustring> expression;
     Gtk::TreeModelColumn<int> ignore_count;
-    Gtk::TreeModelColumn<IDebugger::BreakPoint> breakpoint;
+    Gtk::TreeModelColumn<IDebugger::Breakpoint> breakpoint;
     Gtk::TreeModelColumn<bool> is_standard;
 
+    enum INDEXES {
+        ENABLE_INDEX = 0,
+        ID_INDEX,
+        FILENAME_INDEX,
+        LINE_INDEX,
+        FUNCTION_INDEX,
+        ADDRESS_INDEX,
+        CONDITION_INDEX,
+	IS_COUNTPOINT_INDEX,
+        TYPE_INDEX,
+        HITS_INDEX,
+        EXPRESSION_INDEX,
+        IGNORE_COUNT_INDEX
+    };
+
     BPColumns ()
     {
         add (id);
@@ -60,6 +77,7 @@ struct BPColumns : public Gtk::TreeModelColumnRecord {
         add (line);
         add (breakpoint);
         add (condition);
+	add (is_countpoint);
         add (type);
         add (hits);
         add (expression);
@@ -81,7 +99,7 @@ public:
     Glib::RefPtr<Gtk::ListStore> list_store;
     Gtk::Widget *breakpoints_menu;
     sigc::signal<void,
-                 const IDebugger::BreakPoint&> go_to_breakpoint_signal;
+                 const IDebugger::Breakpoint&> go_to_breakpoint_signal;
     Glib::RefPtr<Gtk::ActionGroup> breakpoints_action_group;
     IWorkbench& workbench;
     IPerspective& perspective;
@@ -100,7 +118,7 @@ public:
         init_actions ();
         build_tree_view ();
         void set_breakpoints
-                (const std::map<int, IDebugger::BreakPoint> &a_breakpoints);
+                (const std::map<int, IDebugger::Breakpoint> &a_breakpoints);
 
         // update breakpoint list when debugger indicates that the list of
         // breakpoints has changed.
@@ -123,56 +141,79 @@ public:
         tree_view->get_selection ()->set_mode (Gtk::SELECTION_MULTIPLE);
 
         //create the columns of the tree view
+	int nb_columns;
         tree_view->append_column_editable ("", get_bp_cols ().enabled);
         tree_view->append_column (_("ID"), get_bp_cols ().id);
         tree_view->append_column (_("Filename"), get_bp_cols ().filename);
         tree_view->append_column (_("Line"), get_bp_cols ().line);
         tree_view->append_column (_("Function"), get_bp_cols ().function);
         tree_view->append_column (_("Address"), get_bp_cols ().address);
-        tree_view->append_column (_("Condition"), get_bp_cols ().condition);
+        tree_view->append_column_editable (_("Condition"),
+                                           get_bp_cols ().condition);
+        tree_view->append_column_editable (_("Toggle countpoint"),
+                                           get_bp_cols ().is_countpoint);
         tree_view->append_column (_("Type"), get_bp_cols ().type);
         tree_view->append_column (_("Hits"), get_bp_cols ().hits);
         tree_view->append_column (_("Expression"),
                                   get_bp_cols ().expression);
-        tree_view->append_column_editable (_("Ignore count"),
-                                           get_bp_cols ().ignore_count);
+        nb_columns = tree_view->append_column_editable (_("Ignore count"),
+							get_bp_cols ().ignore_count);
+	
+	for (int i = 0; i < nb_columns; ++i) {
+            Gtk::TreeViewColumn *col = tree_view->get_column (i);
+            col->set_clickable ();
+            col->set_resizable ();
+            col->set_reorderable ();
+	}
 
         Gtk::CellRendererToggle *enabled_toggle =
             dynamic_cast<Gtk::CellRendererToggle*>
-                                    (tree_view->get_column_cell_renderer (0));
+            (tree_view->get_column_cell_renderer (BPColumns::ENABLE_INDEX));
         if (enabled_toggle) {
             enabled_toggle->signal_toggled ().connect
                 (sigc::mem_fun
                  (*this,
                   &BreakpointsView::Priv::on_breakpoint_enable_toggled));
         }
+	Gtk::CellRendererToggle *countpoint_toggle =
+	  dynamic_cast<Gtk::CellRendererToggle*>
+	  (tree_view->get_column_cell_renderer (BPColumns::IS_COUNTPOINT_INDEX));
+	if (countpoint_toggle) {
+	  countpoint_toggle->signal_toggled ().connect
+	    (sigc::mem_fun (*this,
+			    &BreakpointsView::Priv::on_countpoint_toggled));
+	  
+	}
 
         Gtk::CellRendererText *r =
             dynamic_cast<Gtk::CellRendererText*>
-                (tree_view->get_column_cell_renderer (10));
+            (tree_view->get_column_cell_renderer (BPColumns::IGNORE_COUNT_INDEX));
         r->signal_edited ().connect
             (sigc::mem_fun
              (*this,
               &BreakpointsView::Priv::on_breakpoint_ignore_count_edited));
-        Gtk::TreeViewColumn *c = tree_view->get_column (10);
-        THROW_IF_FAIL (c);
-        c->add_attribute (r->property_editable (),
-                          get_bp_cols ().is_standard);
+
+        r = dynamic_cast<Gtk::CellRendererText*>
+            (tree_view->get_column_cell_renderer (BPColumns::CONDITION_INDEX));
+        r->signal_edited ().connect
+	  (sigc::mem_fun
+	   (*this, &BreakpointsView::Priv::on_breakpoint_condition_edited));
 
         // we must handle the button press event before the default button
         // handler since there are cases when we need to prevent the default
         // handler from running
         tree_view->signal_button_press_event ().connect
-                (sigc::mem_fun
-                     (*this, &Priv::on_breakpoints_view_button_press_signal),
-                      false /*connect before*/);
-        tree_view->get_selection ()->signal_changed ().connect (sigc::mem_fun
-                (*this, &Priv::on_treeview_selection_changed));
+            (sigc::mem_fun
+             (*this, &Priv::on_breakpoints_view_button_press_signal),
+             false /*connect before*/);
+        tree_view->get_selection ()->signal_changed ().connect
+	  (sigc::mem_fun
+	   (*this, &Priv::on_treeview_selection_changed));
 
         tree_view->signal_key_press_event ().connect (sigc::mem_fun
-              (*this, &Priv::on_key_press_event));
+                                                      (*this, &Priv::on_key_press_event));
         tree_view->signal_expose_event ().connect_notify (sigc::mem_fun
-               (*this, &Priv::on_expose_event));
+                                                          (*this, &Priv::on_expose_event));
     }
 
     bool should_process_now ()
@@ -185,7 +226,7 @@ public:
     }
 
     void set_breakpoints
-        (const std::map<int, IDebugger::BreakPoint> &a_breakpoints)
+        (const std::map<int, IDebugger::Breakpoint> &a_breakpoints)
     {
         if (a_breakpoints.empty ()) {
             return;
@@ -197,7 +238,7 @@ public:
             add_breakpoints (a_breakpoints);
         } else {
             //find breakpoints that need adding or updating
-            std::map<int, IDebugger::BreakPoint>::const_iterator breakmap_iter;
+            std::map<int, IDebugger::Breakpoint>::const_iterator breakmap_iter;
             for (breakmap_iter = a_breakpoints.begin ();
                  breakmap_iter != a_breakpoints.end ();
                  ++breakmap_iter) {
@@ -217,11 +258,11 @@ public:
     }
 
     void add_breakpoints
-                (const std::map<int, IDebugger::BreakPoint> &a_breakpoints)
+                (const std::map<int, IDebugger::Breakpoint> &a_breakpoints)
     {
         THROW_IF_FAIL (list_store);
 
-        std::map<int, IDebugger::BreakPoint>::const_iterator break_iter;
+        std::map<int, IDebugger::Breakpoint>::const_iterator break_iter;
         for (break_iter = a_breakpoints.begin ();
                 break_iter != a_breakpoints.end ();
                 ++break_iter) {
@@ -230,10 +271,10 @@ public:
     }
 
     bool breakpoint_list_has_id
-        (const std::map<int, IDebugger::BreakPoint> &a_breakpoints,
+        (const std::map<int, IDebugger::Breakpoint> &a_breakpoints,
          int a_id)
     {
-        std::map<int, IDebugger::BreakPoint>::const_iterator breakmap_iter;
+        std::map<int, IDebugger::Breakpoint>::const_iterator breakmap_iter;
         for (breakmap_iter = a_breakpoints.begin ();
                 breakmap_iter != a_breakpoints.end (); ++ breakmap_iter) {
             if (a_id == breakmap_iter->second.number ()) {
@@ -244,7 +285,7 @@ public:
     }
 
     Gtk::TreeModel::iterator find_breakpoint_in_model
-                                (const IDebugger::BreakPoint &a_breakpoint)
+                                (const IDebugger::Breakpoint &a_breakpoint)
     {
         THROW_IF_FAIL (list_store);
 
@@ -261,39 +302,47 @@ public:
     }
 
     void update_breakpoint (Gtk::TreeModel::iterator& a_iter,
-                            const IDebugger::BreakPoint &a_breakpoint)
+                            const IDebugger::Breakpoint &a_breakpoint)
     {
         (*a_iter)[get_bp_cols ().breakpoint] = a_breakpoint;
         (*a_iter)[get_bp_cols ().enabled] = a_breakpoint.enabled ();
         (*a_iter)[get_bp_cols ().id] = a_breakpoint.number ();
         (*a_iter)[get_bp_cols ().function] = a_breakpoint.function ();
-        (*a_iter)[get_bp_cols ().address] = a_breakpoint.address ();
+        (*a_iter)[get_bp_cols ().address] =
+                                (a_breakpoint.address ().empty ())
+                                ? "<PENDING>"
+                                : a_breakpoint.address ().to_string ();
         (*a_iter)[get_bp_cols ().filename] = a_breakpoint.file_name ();
         (*a_iter)[get_bp_cols ().line] = a_breakpoint.line ();
         (*a_iter)[get_bp_cols ().condition] = a_breakpoint.condition ();
         (*a_iter)[get_bp_cols ().expression] = a_breakpoint.expression ();
         (*a_iter)[get_bp_cols ().ignore_count] =
                                         a_breakpoint.ignore_count ();
-
         (*a_iter)[get_bp_cols ().is_standard] = false;
+        (*a_iter)[get_bp_cols ().is_countpoint] =
+            debugger->is_countpoint (a_breakpoint);
+
         switch (a_breakpoint.type ()) {
-            case IDebugger::BreakPoint::STANDARD_BREAKPOINT_TYPE:
-                (*a_iter)[get_bp_cols ().type] = _("breakpoint");
-                (*a_iter)[get_bp_cols ().is_standard] = true;
-                break;
-            case IDebugger::BreakPoint::WATCHPOINT_TYPE:
-                (*a_iter)[get_bp_cols ().type] = _("watchtpoint");
-                break;
-            default:
-                (*a_iter)[get_bp_cols ().type] = _("unknown");
+	case IDebugger::Breakpoint::STANDARD_BREAKPOINT_TYPE:
+	  (*a_iter)[get_bp_cols ().type] = _("breakpoint");
+	  (*a_iter)[get_bp_cols ().is_standard] = true;
+	  break;
+	case IDebugger::Breakpoint::WATCHPOINT_TYPE:
+	  (*a_iter)[get_bp_cols ().type] = _("watchpoint");
+	  break;
+	case IDebugger::Breakpoint::COUNTPOINT_TYPE:
+	  (*a_iter)[get_bp_cols ().type] = _("countpoint");
+	  break;
+	default:
+	  (*a_iter)[get_bp_cols ().type] = _("unknown");
         }
         (*a_iter)[get_bp_cols ().hits] = a_breakpoint.nb_times_hit ();
     }
 
     Gtk::TreeModel::iterator append_breakpoint
-                                    (const IDebugger::BreakPoint &a_breakpoint)
+                                    (const IDebugger::Breakpoint &a_breakpoint)
     {
-        Gtk::TreeModel::iterator tree_iter = list_store->append();
+        Gtk::TreeModel::iterator tree_iter = list_store->append ();
         update_breakpoint (tree_iter, a_breakpoint);
 
         return tree_iter;
@@ -403,7 +452,7 @@ public:
     }
 
     void on_debugger_breakpoints_set_signal
-                            (const map<int, IDebugger::BreakPoint> &a_breaks,
+                            (const map<int, IDebugger::Breakpoint> &a_breaks,
                              const UString &a_cookie)
     {
         NEMIVER_TRY
@@ -443,7 +492,7 @@ public:
     }
 
     void on_debugger_breakpoint_deleted_signal
-            (const IDebugger::BreakPoint &a_break, int a_break_number,
+            (const IDebugger::Breakpoint &a_break, int a_break_number,
              const UString &a_cookie)
     {
         if (a_break.number () || a_cookie.empty()) {}
@@ -609,6 +658,26 @@ public:
         NEMIVER_CATCH
     }
 
+    void on_countpoint_toggled (const Glib::ustring& path)
+    {
+        NEMIVER_TRY;
+
+        THROW_IF_FAIL (tree_view);
+        Gtk::TreeModel::iterator tree_iter =
+            tree_view->get_model ()->get_iter (path);
+        
+        if (tree_iter) {
+            if ((*tree_iter)[get_bp_cols ().enabled]) {
+                debugger->enable_countpoint
+                    ((*tree_iter)[get_bp_cols ().id], true);
+            } else {
+                debugger->enable_countpoint
+                    ((*tree_iter)[get_bp_cols ().id], false);
+            }
+        }
+        NEMIVER_CATCH;
+    }
+
     void on_breakpoint_ignore_count_edited (const Glib::ustring &a_path,
                                             const Glib::ustring &a_text)
     {
@@ -621,8 +690,8 @@ public:
 
         bool is_standard_bp = false; //true if this is e.g. no watchpoint.
         if (it
-            && (((IDebugger::BreakPoint)(*it)[get_bp_cols ().breakpoint])).
-                type () == IDebugger::BreakPoint::STANDARD_BREAKPOINT_TYPE) {
+            && (((IDebugger::Breakpoint)(*it)[get_bp_cols ().breakpoint])).
+                type () == IDebugger::Breakpoint::STANDARD_BREAKPOINT_TYPE) {
             is_standard_bp = true;
             LOG_DD ("breakpoint is standard");
         } else {
@@ -637,6 +706,26 @@ public:
         NEMIVER_CATCH
     }
 
+    void on_breakpoint_condition_edited (const Glib::ustring &a_path,
+                                         const Glib::ustring &a_text)
+    {
+        NEMIVER_TRY
+
+        Gtk::TreeModel::iterator it = tree_view->get_model ()->get_iter (a_path);
+
+        bool is_standard_bp =
+            (((IDebugger::Breakpoint)(*it)[get_bp_cols ().breakpoint]).type ()
+             == IDebugger::Breakpoint::STANDARD_BREAKPOINT_TYPE)
+            ? true
+            : false;
+
+        if (is_standard_bp)
+            debugger->set_breakpoint_condition ((*it)[get_bp_cols ().id],
+                                                a_text);
+
+        NEMIVER_CATCH
+    }
+
 };//end class BreakpointsView::Priv
 
 BreakpointsView::BreakpointsView (IWorkbench& a_workbench,
@@ -662,7 +751,7 @@ BreakpointsView::widget () const
 
 void
 BreakpointsView::set_breakpoints
-                (const std::map<int, IDebugger::BreakPoint> &a_breakpoints)
+                (const std::map<int, IDebugger::Breakpoint> &a_breakpoints)
 {
     THROW_IF_FAIL (m_priv);
     m_priv->set_breakpoints (a_breakpoints);
@@ -685,7 +774,7 @@ BreakpointsView::re_init ()
     m_priv->re_init ();
 }
 
-sigc::signal<void, const IDebugger::BreakPoint&>&
+sigc::signal<void, const IDebugger::Breakpoint&>&
 BreakpointsView::go_to_breakpoint_signal () const
 {
     THROW_IF_FAIL(m_priv);
